<?php
/**
 * Plugin Name: Cobran&ccedil;a F2b - WooCommerce
 * Plugin URI: https://www.f2b.com.br/desenvolvedores
 * Description: A F2b disponibiliza a integra&ccedil;&atilde;o da Cobran&ccedil;a com o WooCommerce.
 * Author: F2b
 * Author URI: https://www.f2b.com.br/
 * Version: 2.1 
 * License: F2b
 */

add_action( 'plugins_loaded', 'init_f2b_gateway_class' );


function init_f2b_gateway_class(){
	
	add_filter( 'woocommerce_payment_gateways', 'add_f2b_gateway_class' , 1, 10);
	
	function add_f2b_gateway_class( $methods ) {
		$methods[] = 'WC_Gateway_F2b';
		
		return $methods;
	}
	
	class WC_Gateway_F2b extends WC_Payment_Gateway {

		public function __construct() {
			global $woocommerce;
			
			$this->id					= 'f2b';
			$this->icon					= plugins_url( 'image/f2b.gif', __FILE__ );
			$this->has_fields			= false;
			$this->method_title			= __( 'F2b', 'woocommerce' );
			$this->method_description	= __( 'F2b - Meios de Pagamento', 'woocommerce' );
			$this->url      			= 'https://www.f2b.com.br/BillingWeb?'; // Production
			$this->url_status      		= 'https://www.f2b.com.br/api/v1/cobrancas/';
			
			//Load the settings.
			$this->init_form_fields();
			$this->init_settings();
			
			// Define user set variables
			$this->title       = $this->get_option( 'title' );
			$this->description = $this->get_option( 'description' );
			
			// Define user set variables
			$this->conta 			 = $this->get_option( 'conta' );
			$this->tipo_cobranca	 = $this->get_option( 'tipo_cobranca' );
			$this->prazo_vencimento  = $this->get_option( 'prazo_vencimento' );
			$this->multa_valor  	 = $this->get_option( 'multa_valor' );
			$this->tipo_multa  	 	 = $this->get_option( 'tipo_multa' );
			$this->multa_valor_dia   = $this->get_option( 'multa_valor_dia' );
			$this->tipo_multa_dia  	 = $this->get_option( 'tipo_multa_dia' );
			$this->multa_atraso  	 = $this->get_option( 'multa_atraso' );
			$this->demonstrativo_1 	 = $this->get_option( 'demonstrativo_1' );
			$this->token_f2b         = $this->get_option( 'token_api' );
			$this->envio 			 = $this->get_option( 'envio' );
			$this->debug             = $this->get_option( 'debug' );
			
			// Active logs.
			if ( 'yes' === $this->debug ) {
				if ( function_exists( 'wc_get_logger' ) ) {
					$this->log = wc_get_logger();
				} else {
					$this->log = new WC_Logger();
				}
			}
				
			// Actions
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_api_f2b', array( $this, 'payment_update' ) );
		}
		
		/**
		 * Format number.
		 *
		 * @param  string $string Number to format.
		 *
		 * @return string
		 */
		protected function format_number( $string ) {
			return str_replace( array( '.', '-', '/' ), '', $string );
		}		
		
		public function admin_options() {

            echo '<h3>' . __( 'Cobran&ccedil;a F2b', 'woocommerce' ) . '</h3>';

            if ( ! $this->is_valid_for_use() ) {

                // Valid currency.
                echo '<div class="inline error"><p>'. __( 'Mude a moeda da loja para Reais (R$).', 'woocommerce' ) . '</p></div>';

            } else {

                // Generate the HTML For the settings form.
                echo '<table class="form-table">';
                $this->generate_settings_html();
                echo '</table>';
            }
        }
		/**
		 * Get log.
		 *
		 * @return string
		 */
		protected function get_log_view() {
			if ( defined( 'WC_VERSION' ) && version_compare( WC_VERSION, '2.2', '>=' ) ) {
				return '<a href="' . esc_url( admin_url( 'admin.php?page=wc-status&tab=logs&log_file=' . esc_attr( $this->id ) . '-' . sanitize_file_name( wp_hash( $this->id ) ) . '.log' ) ) . '">' . __( 'System Status &gt; Logs', 'woocommerce' ) . '</a>';
			}

			return '<code>woocommerce/logs/' . esc_attr( $this->id ) . '-' . sanitize_file_name( wp_hash( $this->id ) ) . '.txt</code>';
		}		
		function is_valid_for_use() {
			if ( ! in_array( get_woocommerce_currency(), array( 'BRL' ) ) ) return false;
			return true;
		}
		function init_form_fields(){
			$this->form_fields = array(
				'enabled' => array(
					'title' => __( 'Habilitar/Desabilitar', 'woocommerce' ),
					'type' => 'checkbox',
					'label' => __( 'Habilitar meio de pagamento F2b', 'woocommerce' ),
					'default' => 'yes'
				),
				'title' => array(
					'title' => __( 'T&iacute;tulo', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'T&iacute;tulo do Checkout', 'woocommerce' ),
					'default' => __( 'Cobran&ccedil;a F2b', 'woocommerce' )
				),
				'description' => array(
					'title' => __( 'Descrição', 'woocommerce' ),
					'type' => 'textarea',
					'description' => __( 'Descrição que será exibida no Checkout', 'woocommerce' ),
					'default' => __( 'Pagar através de Boleto ou Pix', 'woocommerce-moip' )
				),				
				'conta' => array(
					'title' => __( 'N&uacute;mero da Conta', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Digite o n&ordm; de sua conta na F2b sem espa&ccedil;os. Geralmente come&ccedil;a com 9023... e possui 16 d&iacute;gitos.', 'woocommerce' ),
					'default' => ''
				),
				'token_api' => array(
					'title' => __( 'Token API da F2b', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Preencha o token da API na F2b. Acesse sua conta no menu Conta - Segurança - Web Services', 'woocommerce' ),
					'default' => ''
				),
				'tipo_cobranca' => array(
					'title' => __( 'Meios de Pagto Aceitos', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'B - Boleto; P - Pix;. Ex.: BP (Aceitar Boleto, Pix)', 'woocommerce' ),
					'default' => ''
				),
				'prazo_vencimento' => array(
					'title' => __( 'Prazo para Vencimento', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Prazo em dias para o vencimento. O vencimento ser&aacute; a data atual somada ao n&uacute;mero de dias do prazo de vencimento.', 'woocommerce' ),
					'default' => ''
				),
				'multa_valor' => array(
					'title' => __( 'Valor da multa', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Valor em Reais ou Porcentagem da multa aplicada no caso de pagamento em atraso. Ex. 5.00', 'woocommerce' ),
					'default' => ''
				),
				'tipo_multa' => array(
					'title' => __( 'Tipo da multa', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Tipo da multa. 0 = R$ e 1 = %', 'woocommerce' ),
					'default' => ''
				),
				'multa_valor_dia' => array(
					'title' => __( 'Valor da multa diária', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Valor em Reais ou Porcentagem da multa diária aplicada no caso de pagamento em atraso. Ex. 0.20', 'woocommerce' ),
					'default' => ''
				),
				'tipo_multa_dia' => array(
					'title' => __( 'Tipo da multa diária', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Tipo da multa diária. 0 = R$ e 1 = %', 'woocommerce' ),
					'default' => ''
				),
				'multa_atraso' => array(
					'title' => __( 'Prazo máximo para pagamento', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'N&uacute;mero de dias após o vencimento at&eacute; quando ser&aacute; aceito o pagamento em atraso', 'woocommerce' ),
					'default' => ''
				),
				'demonstrativo_1' => array(
					'title' => __( 'Demonstrativo', 'woocommerce' ),
					'type' => 'text',
					'description' => '',
					'default' => ''
				),
				'envio' => array(
					'title' => __( 'Tipo de envio', 'woocommerce' ),
					'type' => 'text',
					'description' => __( 'Preencha como deseja o envio da Cobran&ccedil;a F2b. e - Email; n - Nenhum (Padr&atilde;o)', 'woocommerce' ),
					'default' => ''
				),
				'debug'           => array(
					'title'       => __( 'Debug Log', 'woocommerce' ),
					'type'        => 'checkbox',
					'label'       => __( 'Habilitar logs', 'woocommerce' ),
					'default'     => 'no',
					'description' => sprintf( __( 'Logs dos eventos F2b, como requests de API. %s', 'woocommerce' ), $this->get_log_view() ),
				),				
			);
		}
		function process_payment( $order_id ) {
			global $woocommerce;
			$order = new WC_Order( $order_id );
			
			$f2b_args = $this->get_f2b_args( $order );
			
			$f2b_args = http_build_query( $f2b_args, '', '&' );
			
			$f2b_url = $this->url;

			// Mark as on-hold (we're awaiting the cheque)
			$order->update_status('on-hold', __( 'Aguardando pagamento (F2b)', 'woocommerce' ));

			// Reduce stock levels
			$order->reduce_order_stock();

			// Remove cart
			$woocommerce->cart->empty_cart();

			// Return thankyou redirect
			return array(
				'result' => 'success',
				//'redirect'  => $this->get_return_url( $order )
				'redirect' => $f2b_url . $f2b_args
			);
		}
		function receipt_page( $order ) {

			echo '<p>'.__( 'Obrigado pelo pedido, por favor clique no bot&atilde;o abaixo para pagar com a F2b.', 'woocommerce' ).'</p>';

			echo $this->generate_f2b_form( $order );

		}
		function get_f2b_args( $order ) {
			global $woocommerce;
			
			$order_id = $order->id;
			
			$wcbcf_settings = get_option( 'wcbcf_settings' );
			$wcbcf_settings = isset( $wcbcf_settings['person_type'] ) ? intval( $wcbcf_settings['person_type'] ) : 0;
			
			$nome_cliente = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();
			$cpf = '';
			$cnpj = '';
			
			if ( ( 0 === $wcbcf_settings || 2 === $wcbcf_settings ) && '' !== $order->get_meta( '_billing_cpf' ) ) {
				$cpf = $this->format_number( $order->get_meta( '_billing_cpf' ) );
				$nome_cliente = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();
			} else if ( ( 0 === $wcbcf_settings || 3 === $wcbcf_settings ) && '' !== $order->get_meta( '_billing_cnpj' ) ) {
				$cnpj = $this->format_number( $order->get_meta( '_billing_cnpj' ) );
				$nome_cliente = $order->get_billing_company();
			} else if ( '' !== $order->get_meta( '_billing_persontype' ) ) {
				if ( 1 === intval( $order->get_meta( '_billing_persontype' ) ) && '' !== $order->get_meta( '_billing_cpf' ) ) {
					$cpf = $this->format_number( $order->get_meta( '_billing_cpf' ) );
					$nome_cliente = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();
				} else if ( 2 === intval( $order->get_meta( '_billing_persontype' ) ) && '' !== $order->get_meta( '_billing_cnpj' ) ) {
					$cnpj = $this->format_number( $order->get_meta( '_billing_cnpj' ) );
					$nome_cliente = $order->get_billing_company();
				}
			}
			
			$f2b_args = array(
				'conta'					=> $this->conta,
				'tipo_cobranca'			=> $this->tipo_cobranca,
				'prazo_vencimento' 		=> $this->prazo_vencimento,
				'multa_valor' 			=> $this->multa_valor,
				'tipo_multa' 			=> $this->tipo_multa,
				'multa_valor_dia' 		=> $this->multa_valor_dia,
				'tipo_multa_dia' 		=> $this->tipo_multa_dia,
				'multa_atraso' 			=> $this->multa_atraso,
				'demonstrativo_1'		=> $this->demonstrativo_1,
				'demonstrativo_2'		=> 'Número do pedido: #' . $order->id,
				// Billing info
				'nome'					=> $nome_cliente,
				'email_1'				=> $order->billing_email,
				'endereco_logradouro'	=> $order->billing_address_1,
				'endereco_numero'		=> $order->get_meta( '_billing_number' ),
				'endereco_complemento'	=> $order->billing_address_2,
				'endereco_bairro'		=> $order->get_meta( '_billing_neighborhood' ),
				'endereco_cidade'		=> $order->billing_city,
				'endereco_estado'		=> $order->billing_state,
				'endereco_cep'			=> $order->billing_postcode,
				'envio'					=> $this->envio,
				'cpf'					=> $cpf,
				'cnpj'					=> $cnpj,
				
				'num_document'		    => $order->id,
				'tipo_ws'				=> 'woocommerce',
				
				'valor'				=> number_format( $order->get_total(), 2, '.', '' )
				
			);
			
			$f2b_args = apply_filters( 'woocommerce_f2b_args', $f2b_args );

			return $f2b_args;
		}
		function generate_f2b_form( $order_id ) {
			global $woocommerce;
			
			$order = new WC_Order( $order_id );
			
			$f2b_url = $this->url;
			
			$f2b_args = $this->get_f2b_args( $order );

			$f2b_args_array = array();
			
			foreach ($f2b_args as $key => $value) {
				$f2b_args_array[] = '<input type="hidden" name="'.esc_attr( $key ).'" value="'.esc_attr( $value ).'" />';
			}
			
			$woocommerce->add_inline_js( '
				jQuery("body").block({
					message: "' . esc_js( __( 'Obrigado pelo seu pedido. Agora estamos redirecionando voc&ecirc; para a F2b para fazer o pagamento.', 'woocommerce' ) ) . '",
					baseZ: 99999,
					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},
					css: {
						padding:        "20px",
						zindex:         "9999999",
						textAlign:      "center",
						color:          "#555",
						border:         "3px solid #aaa",
						backgroundColor:"#fff",
						cursor:         "wait",
						lineHeight:		"24px",
					}
				});
				jQuery("#submit_f2b_payment_form").click();
			' );
			
			return '<form action="'.esc_url( $f2b_url ).'" method="post" id="f2b_payment_form" target="_top">
					' . implode( '', $f2b_args_array) . '
					<input type="submit" class="button alt" id="submit_f2b_payment_form" value="' . __( 'Pagamento via F2b', 'woocommerce' ) . '" /> <a class="button cancel" href="'.esc_url( $order->get_cancel_order_url() ).'">'.__( 'Cancel pedido', 'woocommerce' ).'</a>
					</form>';
		}
		
		public function payment_update() {
		 
			$numero_cobranca = '';
			$numero_documento = '';
			foreach ($_GET as $key => $value) {
				$value = urlencode(stripslashes($value));
				if ($key == 'num_billing') {
					$numero_cobranca = $value;
				}
				if ($key == 'num_document') {
					$numero_documento = $value;
				}
			}
			
			$ch = curl_init($this->url_status . $numero_cobranca);
		
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");
			
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(
				'Content-Type: application/json; charset=utf-8',
				'Authorization: Basic ' . base64_encode($this->token_f2b)
				)
			);
			
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			
			$output = curl_exec($ch);
			$info = curl_getinfo($ch);
			if($errno = curl_errno($ch)) {
				if ( 'yes' === $this->debug ) {
					$this->log->add( $this->id, 'Erro: ' . $errno . ' - ' . curl_strerror($errno));
				}
			} else {
				$cobranca = json_decode($output, true);
			
				if ($output === true || $info['http_code'] == 200) {
					
					foreach($cobranca as $key => $value){
						if ($key == "codigo_status")
							$situacao = $value;
						if ($key == "pagamentos"){
							
							foreach($cobranca[$key] as $key2 => $value2){
								foreach($value2 as $key3 => $value3){
									if ($key3 == "valor_pagamento")
										$valor = $value3;
									if ($key3 == "codigo_tipo_pagamento")
										$tipo_pagamento = $value3;
									if ($key3 == "taxa_pagamento")
										$taxa_pagamento = $value3;
									if ($key3 == "data_pagamento") {
										$day = substr($value3,0,2);
										$month = substr($value3,3,2);
										$year = substr($value3,6,4);
										$data_pagamento = $year . "-" . $month . "-" . $day;
									}
								}
							}
						}
					}
					
					if ($situacao==2||$situacao==5||$situacao==6||$situacao==7) {
						$order = new WC_Order( $numero_documento );
						$order->add_order_note( __( 'F2b: Pagamento aprovado.', 'woocommerce' ) );
						$order->payment_complete();
					}
					
				}else{
					foreach( $cobranca['erro'] as $key => $value ) {
						if (strcmp($key, "mensagens") !== 0){
							if ( 'yes' === $this->debug ) {
								$this->log->add( $this->id, 'Erro: ' . $value);
							}
							
						}else{
							foreach($value as $msg){
								if ( 'yes' === $this->debug ) {
									$this->log->add( $this->id, 'Erro: ' . $msg);
								}
							}
						}
					}
				}
			}
			curl_close($ch);
		}
	}

}
?>
